/**
 * Copyright eLearning Brothers LLC 2012 All Rights Reserved
 */
var game = new function () {
    var designFile = "design.ini.txt";
    var questionsFile = "questions.ini.txt";
    var soundsFile = "sounds.ini.txt";

    if (!empty(GAMEPREFIX)) {
        soundsFile = GAMEPREFIX + "-sounds.ini.txt";
        questionsFile = GAMEPREFIX + "-questions.ini.txt";
        designFile = GAMEPREFIX + "-design.ini.txt";
    }

    var questions,
        questionCount = 0,
        design,
        sounds,
        validity = -3,
        instance = this,
        score = 0,
        answers = {},
        questionIndex = 0,
        gameScore = 0,
        gameFee = 0,
        error = 0,
        scoreQuestion = 0,
        timerClick,
        timeout = 40,
        arrow = 0,
        light_obj = 0,
        old_section = 0,
        numberOfSectionPass = 0,
        counter_move_ie = 0,
        benchmark_result = 0,
        statistic = {},
        start_time,
        current_time,
        quiz_percent = 0,
        currentQuestionSound = null;


    /* -----------------------  LOADING ------------------------- */

    this.reloadStyles = function () {
        $.get("config/" + designFile, function (iniData) {
            design = parseIni(iniData);
            loadStyles();
        });
    };

    this.readConfig = function () {
        var prev = new Date().getTime();
        var new_time = prev;
        var temp_variable = 0;
        for (i = 0; (new_time - prev) < 500; i++) {
            temp_variable = Math.floor(((Math.random() * 5000) ^ 2) / 0.49574957952);
            benchmark_result++;
            new_time = new Date().getTime();
        }
        //console.log("benchmark_result",benchmark_result);


        $.get("config/" + questionsFile, function (iniData) {
            iniData += prepareIni(iniData, standartQuestionPattern);
            questions = parseIni(iniData);
            setOriginalQuestions(questions);
            defaultQuestionPostProcesor(questions);
            validity++;
            if (validity == 0) {
                $(document).trigger('gameLoaded');
            }
        });
        $.get("config/" + designFile, function (iniData) {
            design = parseIni(iniData);
            validity++;
            if (validity == 0) {
                $(document).trigger('gameLoaded');
            }
        });
        $.get("config/" + soundsFile, function (iniData) {
            sounds = parseIni(iniData, function (val) {
                return createSound(val);
            });
            validity++;
            if (validity == 0) {
                $(document).trigger('gameLoaded');
            }
        });

    };

    var loadStyles = function () {
        instance.loadStyles();
    };
    var bindSounds = function () {
        instance.loadSounds();
    };
    var fillData = function () {
        instance.loadData();
    };

    $(document).bind('gameLoaded', function () {
        loadStyles();
        bindSounds();
        fillData();

        $('div.game').addClass('step-1');

        instance.onGameLoaded();

    });


    /* -----------------------  FUNCTIONS ------------------------- */


    function recalculateScore() {
        var answeredCount = 0;
        var correctAnsweredCount = 0;
        var answeredScore = 0;
        for (var k in answers) {
            answeredCount++;
            answeredScore += answers[k];
            if (answers[k] > 0) correctAnsweredCount++;
        }
        gameScore = answeredScore;
        instance.onUpdateScore(gameScore);
        instance.updateQuizPercent(correctAnsweredCount);
    }

    this.updateQuizPercent = function (count) {
        statistic.game_percent = (100 * statistic.correct_answers / (statistic.incorrect_answers + statistic.correct_answers)).toFixed(0);
//        quiz_percent = (100*count/questionCount).toFixed(2);//old code
        quiz_percent = (100 * statistic.correct_answers / (statistic.incorrect_answers + statistic.correct_answers)).toFixed(2);
        $('div.game .quiz-percent-value').html(quiz_percent + "%");
    };

    /* -----------------------  GAME FLOW START ------------------------- */

    /* -----------------------  STEP-0 game start, reset all ------------------------- */
    this.start = function () {
        questions = getOriginalQuestions();
        defaultQuestionPostProcesor(questions);
        instance.loadData();
        tRewind(sounds.finish);
        tRewind(sounds.conclusion);
        if (!onlyOneSound) {
            setTimeout(function () {
                tRewind(sounds.start);
                tPlay(sounds.start, PRIORITY_NORMAL);
            }, 10);
        }

        statistic.correct_answers = 0;    //every correct answer
        statistic.incorrect_answers = 0;  //every incorrect attempt
        statistic.fail_answers = 0;       //when all attempts was incorrect
        statistic.questions_time = {};
        statistic.questions_answers = {};

        answers = {};
        questionIndex = 0;

        error = 0;
        gameScore = 0;
        gameFee = 0;

        $('div.game #animation').hide();
        //$('.indicator').hide();
        $('div.game div.score').html(0);

        $('#game').removeClass('step-0').addClass('step-1');
        $(document).trigger('gameStarted');
    };

    this.onGameLoaded = function () {
        timeout = parseFloat(questions.rotationSpeed) * 1000;
        // console.log(timeout);
        instance.prepareAnimationFrame();
    };

    /* -----------------------  STEP-1 Logo, Splash screen ------------------------- */

    liveFastClick('div.game a.button-game-start-1', function () {
        $('#game').removeClass('step-1').addClass('step-2');
        if (onlyOneSound) {
            tRewind(sounds.start);
            if (web_audio_api_player.init()) {
                if (sounds.introduction != null) {
                    tPlay(sounds.start, PRIORITY_NORMAL);
                    if ($('#game').hasClass('step-2')) {
                        tRewind(sounds.start, 3, 1);
                        tPlay(sounds.introduction, PRIORITY_NORMAL, 4);
                    }
                }
                else {
                    tPlay(sounds.start, PRIORITY_NORMAL);
                }
            }
            else {
                if (sounds.introduction != null) {
                    tPlay(sounds.introduction, PRIORITY_NORMAL);
                }
                else {
                    tPlay(sounds.start, PRIORITY_NORMAL);
                }
            }
        } else {
            tRewind(sounds.start);
            if (sounds.introduction != null) {
                tPlay(sounds.introduction, PRIORITY_NORMAL);
            }
        }
    });

    /* -----------------------  STEP-2 Intro to game, it's description ------------------------- */

    liveFastClick('div.game #step2continuebutton', function () {
        tRewind(sounds.start);
        tRewind(sounds.introduction);
        $('#game').removeClass('step-2');
        start_time = new Date().getTime();
        questionIndex = 0;
        instance.onQuestionChooseRequired();
    });

    this.onQuestionChooseRequired = function () {
        questionIndex++;
        onQuestionIndexApplied();
    };
    /* -----------------------  STEP-3 Questions choosing board ------------------------- */

    /**
     * New question index is set, show it
     */
    function onQuestionIndexApplied() {
        if (!empty(questions['q' + questionIndex])) {
            $('#game').removeClass('step-3');
            questionShow(questionIndex);
        } else {

            finishGame();
        }
    }


    /**
     * On pressing continue button on question board screen
     */
    liveFastClick('div.game #step3continuebutton', function () {
        onQuestionIndexApplied();
    });

    /**
     * On pressing continue button on question board screen
     */
    liveFastClick('div.game #step4wheelbutton', function () {
        $('div.game #animation').hide();
        $('div.game .buttons-continue-wheel .button').removeClass('show');
        $('div.game .question-block-wrapper').show();
        //$('.indicator').show();
        $('div.game div.question-block-wrapper-inner, .question-choose-wrapper').show();
        if (currentQuestionSound !== null){
            tRewind(currentQuestionSound);
            tPlay(currentQuestionSound, PRIORITY_HI);
        }

    });

    /**
     * Start wheel
     */
    liveFastClick('div.game .wheelspin div.start_button', function () {
        if (!$('div.game .wheelspin').hasClass('not_hover')) {
            tPlay(sounds.select, PRIORITY_LOW);
            if (!$('.buttons-continue-wheel .button').hasClass('show')) {
                if ($(this).hasClass('start')) {
                    $(this).removeClass('start').html('');
                    stopArrowAndAskQuestion();
                    $('div.game .wheelspin').addClass('not_hover');
                    //clearInterval(timerClick);
                    setTimeout(function () {
                        tPlay(sounds.hover, PRIORITY_LOW);
                    }, 2000)
                } else {
                    $(this).addClass('start').html(value(questions.wheel_page_button_stop_text));
                    var ar = ($('div.game .arrow').data('run')) ? $('.arrow').data('run') : [0, 10, 10];
                    if ($.browser.msie) {
                        counter_move_ie = 0;
                        moveArrow(ar[0], ar[1], ar[2], 1, timeout / 36);
                    } else {

                        var transition_time = timeout * 2777;
                        arrow.css({
                            "-webkit-Transform": 'rotate(999999deg) translateZ(0)',
                            "-moz-transform": 'rotate(999999deg)',
                            "-ms-transform": 'rotate(999999deg)',
                            "-o-transform": 'rotate(999999deg)',
                            " transform": 'rotate(999999deg)',
                            "-webkit-Transition-Duration": transition_time + 'ms',
                            "-moz-Transition-Duration": transition_time + 'ms',
                            "-o-Transition-Duration": transition_time + 'ms',
                            "-ms-Transition-Duration": transition_time + 'ms',
                            "transition-duration": transition_time + 'ms'
                        });
                    }
                    old_section = -1;
                    numberOfSectionPass = 0;
                    timerClick = setInterval(moveStars, timeout / 36);

                }
            }
        }
    });


    function moveStars() {
        var current_Deg = getCurrentRotateDeg();
        var current_section = (current_Deg / 60 | 0);//Like Math.floor but faster
        //var current_section=Math.floor(current_Deg/60);
        if (!(current_section == old_section)) {
            numberOfSectionPass++;
            tPlay(sounds.hover, PRIORITY_LOW);
            light_obj.removeClass('glow');
            for (i = 0; i < 6; i++) {
                light_obj.eq((current_section) * 6 + i).addClass('glow');
            }
            old_section = current_section;
        }
    }

    function stopArrowAndAskQuestion() {
        // var prev=new Date().getTime();
        var currentDeg = ((numberOfSectionPass - 1) / 6 | 0) * 360 + getCurrentRotateDeg() + 120;
        // console.log("currentDeg",currentDeg-120);
        //  console.log("currentdeg -", new Date().getTime()-prev);
        //console.log("numberOfSectionPass",numberOfSectionPass,"((numberOfSectionPass-1)/6|0)",((numberOfSectionPass-1)/6|0),"getCurrentRotateDeg",getCurrentRotateDeg(),"currentDeg",currentDeg);
        if (!$.browser.msie) {
            arrow.css({//очищем стиль. Это заставит стрелку остановиться.
                "-moz-Transform": '',
                "-o-Transform": '',
                "-ms-Transform": '',
                "Transform": '',
                "-webkit-Transform": ''
            });
            var timer_time = 0;
            if (!onlyOneSound) {
                if (benchmark_result < 200000) {
                    timer_time = 10;
                } else if (benchmark_result < 185000) {
                    timer_time = 20;
                } else if (benchmark_result < 140000) {
                    timer_time = 30;
                } else if (benchmark_result < 100000) {
                    timer_time = 50;
                }
            }
            //console.log("timer",timer_time);
            setTimeout(function () {
                    currentDeg = ((numberOfSectionPass - 1) / 6 | 0) * 360 + getCurrentRotateDeg() + 120;
                    //  console.log("currentDeg",currentDeg-120);
                    // console.log("currentdeg -", new Date().getTime()-prev);
                    // prev=new Date().getTime();
                    arrow.css({
                        "-moz-Transform": 'rotate(' + (currentDeg) + 'deg) ',
                        "-o-Transform": 'rotate(' + (currentDeg) + 'deg) ',
                        "-ms-Transform": 'rotate(' + (currentDeg) + 'deg) ',
                        "Transform": 'rotate(' + (currentDeg) + 'deg) ',
                        "-webkit-Transform": 'rotate(' + (currentDeg) + 'deg) translateZ(0)',
                        "-webkit-Transition-Duration": timeout / 1 + 'ms',
                        "-webkit-Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)',
                        "-moz-Transition-Duration": timeout / 1 + 'ms',
                        "-o-Transition-Duration": timeout / 1 + 'ms',
                        "-ms-Transition-Duration": timeout / 1 + 'ms',
                        "transition-duration": timeout / 1 + 'ms',
                        "-moz-Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)',
                        "-0-Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)',
                        "-ms-Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)',
                        "Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)'

                    });
                    // currentDeg=((numberOfSectionPass-1)/6|0)*360+getCurrentRotateDeg()+120;
                    // console.log("currentDeg",currentDeg-120);
                    // console.log("currentdeg -", new Date().getTime()-prev);
                    //prev=new Date().getTime();


                    setTimeout(function () {
                        clearInterval(timerClick);
                        currentDeg = getCurrentRotateDeg();
                        scoreQuestion = (currentDeg < 60 || currentDeg >= 240 && currentDeg < 300) ? 300 : (currentDeg >= 60 && currentDeg < 120 || currentDeg >= 180 && currentDeg < 240) ? 500 : 100;
                        //console.log(scoreQuestion);
                        $('#game').data('score', scoreQuestion);
                        $('div.game .buttons-continue-wheel .button').addClass('show');
                        $('div.game .wheelspin').addClass('not_hover');
                    }, timeout / 1);
                },
                timer_time)
        }

    }

    /**
     * No more questions available, go to finish screen, send score to SCROM
     */
    function finishGame() {
        flashBackground('start');

        //$('div.result-block h1').html("" + value(questions.conclusion_title));
        //$('div.result-block div.description').html( value(questions.conclusion_text));

        //var Q = 100-(100/questionCount*error);
        $('div.game .score-description').html(value(questions.results)+' <span>' + quiz_percent + '%</span>');

        $('#game').removeClass('step-3');
        $('#game').removeClass('step-4');
        $('#game').addClass('step-5');

        if (sounds.conclusion != null) {
            tPlay(sounds.conclusion, PRIORITY_NORMAL);
        }
        else {
            tPlay(sounds.finish, PRIORITY_NORMAL);
        }
        try {
            SCOSetValue("score", quiz_percent);
            SCOSetValue("completed", 1);
            SCOCommit();
        } catch (e) {
            console.error("Scorm failed -", e);
        }
        if ('undefined' !== typeof gameloader) {
            gameloader.send_results(statistic);
        }
        $(document).trigger('gameFinished', [statistic]);
    }


    /* -----------------------  STEP-4 Answers ------------------------- */

    /**
     * Show question board for index i
     * @param i
     */
    function questionShow(i) {
        //$('.indicator').hide();
        $('div.game #animation').show();
        $('div.game .question-block-wrapper').hide();
        $('div.game div.question-block-wrapper-inner, .question-choose-wrapper').hide();
        $('div.game .wheelspin div.start_button').html(value(questions.wheel_page_button_spin_text));
        $('div.game .wheelspin').removeClass('not_hover');
        arrow.css({
            "-webkit-Transition-Duration": 0 + 'ms',
            "-webkit-Transform": 'rotate(0deg) translateZ(0) ',
            "-moz-Transition-Duration": 0 + 'ms',
            "-o-Transition-Duration": 0 + 'ms',
            "-ms-Transition-Duration": 0 + 'ms',
            "transition-duration": 0 + 'ms',
            "-moz-Transform": 'rotate(0deg)',
            "-o-Transform": 'rotate(0deg) ',
            "-ms-Transform": 'rotate(0deg) ',
            "Transform": 'rotate(0deg) ',
            "-webkit-Transition-Timing-Function": 'linear',
            "-moz-Transition-Timing-Function": 'linear',
            "-0-Transition-Timing-Function": 'linear',
            "-ms-Transition-Timing-Function": 'linear',
            "Transition-Timing-Function": 'linear'
        }).data('run', [0, 10, 10]);
        $('div.game .light').removeClass('glow');

        $('#game').addClass('step-4');
        var question = questions['q' + i];

        $('div.game .indicator').html(i + '/' + questionCount);

        var score = nvl(question.score, 1);
        /* Question score is optional, default 1 per question */
        current_time = (new Date().getTime());
        if (statistic.questions_answers['q' + i] != 0) statistic.questions_time['q' + i] = 0;//reset question time on start/correct/fail (don't reset on incorrect)
        $('#game').data('question', question).data('score', score);

        $('div.game .step-4').removeClass("type-multiple");
        //conditionalShow($('.question-block h1'),question.title);
        $('div.game .question-block div').html(value(question.text));
        $('div.game .question-choose').html("");
        /* arrow.css({
         "-webkit-Transition-Duration": timeout+'ms',
         "-webkit-Transition-Timing-Function": 'linear',
         "-moz-Transition-Duration": timeout+'ms',
         "-o-Transition-Duration": timeout+'ms',
         "-ms-Transition-Duration": timeout+'ms',
         "transition-duration":  timeout+'ms',
         "-moz-Transition-Timing-Function": 'linear',
         "-0-Transition-Timing-Function": 'linear',
         "-ms-Transition-Timing-Function": 'linear',
         "Transition-Timing-Function": 'linear'});*/

        var correct = value(question.correct_answer).split(',');

        var i = 1;
        var order = [];

        // optionally include eli animbutton and clickalert state when specified by ini settings
        var eliAnimButtonDivOpen = (design.eli_anim_button_enabled) ? "<div class='eli-button'>" : "";
        var eliAnimButtonDivClose = (design.eli_anim_button_enabled) ? "</div>" : "";
        var eliAnimClickAlertDiv = (design.eli_anim_clickalert_enabled) ? "<div class='eli-clickalert'></div>" : "";

        while (!empty(value(question['answer_' + i]))) {
            var variant = $("<div class='variant'>" + eliAnimButtonDivOpen + "<div class='table'><div>" + value(question['answer_' + i]) + "</div></div>" + eliAnimClickAlertDiv + eliAnimButtonDivClose + "</div>");
            variant.data({'correct': false, 'number': i});
            for (var k in correct) {
                if (i == correct[k].trim()) {
                    variant.data('correct', true);
                }
            }
            order[order.length] = variant;
            i++;
        }
        if (questions.randomize_answer_order) {
            order.sort(function () {
                return 0.5 - Math.random()
            });
        }

        for (var k in order) {
            $('div.game .question-choose').append(order[k]);
            //$('.question-choose .variant').eq(k).find('div').prepend(letter[k]+'. ')
        }

        if (!empty(question.type)) {
            $('div.game .step-4').addClass("type-" + value(question.type));
        }
        if (!empty(question.audio)) {
            currentQuestionSound = createSound(value(question.audio), true);
        } else {
            currentQuestionSound = null;
        }
        removeBackgroundApply($('div.game .question-block-wrapper'), question);
        if (!empty(question.image)) {
            var image = value(question.image);
            var removeBackground = true;
            if (!empty(question.removeBackground)) {
                removeBackground |= stringToBoolean(value(question.removeBackground));
            }
            $('div.game .question-block-wrapper>div.question-image').css('background-image', 'url("' + parseImgPath(image, true) + '")');
        } else {
            $('div.game .question-block-wrapper>div.question-image').css('background-image', 'none');
        }
        $('div.game .question-block-wrapper').css({opacity: 0}).animate({opacity: 1}, 'slow');

        $('#game .step-4').removeClass('answered').addClass('unanswered').removeClass('correct').removeClass('incorrect');

        // call core helper to init eli sprite anims
        initEliSpriteAnims(design);
    }

    function questionHide(i) {
        instance.onQuestionHide(i);
    }

    /**
     * Question is hidden, do custom actions
     */
    this.onQuestionHide = function (questionI) {
        onQuestionIndexApplied();
    }

    liveFastClick('div.game div.question-choose .variant', function () {
        $(this).toggleClass('choosed');
        answerChanged();
    });

    /**
     * Answer is changed
     */
    var answerChanged = function () {
        if (!$('div.game div.step-4').hasClass('type-multiple')) {
            answerConfirmed();
        }
    };


    liveFastClick('div.game a.button-question-confirm', function () {
        answerConfirmed();
    });

    /**
     * Answer is confirmed
     */
    var answerConfirmed = function () {
        var question = $('#game').data('question');
        var lscore = $('#game').data('score');
        var answerIndex = -1;
        var answerNumber = 0;
        statistic.questions_time['q' + questionIndex] += (new Date().getTime()) - current_time;

        var allCorrectRequired = $('div.game div.step-4').hasClass('type-multiple');
        var correct = allCorrectRequired;

        $('div.game div.question-choose').find('.variant').each(function () {

            /* If required all correct answers to be choosed */
            if (allCorrectRequired && $(this).hasClass('choosed') != $(this).data('correct')) {
                correct = false;
            }

            /* If required one correct answers to be choosed */
            if (!allCorrectRequired && $(this).hasClass('choosed') && $(this).data('correct')) {
                correct = true;
            }

            if ($(this).hasClass('choosed')) {
                answerIndex = $(this).index();
                answerNumber=$(this).data('number');
            }
        });

        if (correct) {

            tRewind(sounds.correct);
            tPlay(sounds.correct, PRIORITY_NORMAL);
            answers[questionIndex] = parseInt(lscore);
            statistic.questions_answers['q' + questionIndex] = 1;
            statistic.correct_answers++;

            $('div.game .question-answered-block div').html((allCorrectRequired) ? value(question.correct_feedback_text) : value(question['answer_' + answerNumber].feedback_text));
            $('#game .step-4').addClass('correct');
            $('div.game #step4continuebutton').show();

            $('#game .step-4').removeClass('unanswered').addClass('answered');

        } else {

            error++;

            $('div.game .question-answered-block div').html((allCorrectRequired) ? value(question.incorrect_feedback_text) : value(question['answer_' + answerNumber].feedback_text));
            $('#game .step-4').addClass('incorrect');
            $('#game .step-4').removeClass('unanswered').addClass('answered');
            $('div.game #step4continuebutton').show();

            tRewind(sounds.incorrect);
            tPlay(sounds.incorrect, PRIORITY_NORMAL);
            statistic.questions_answers['q' + questionIndex] = 0;
            statistic.incorrect_answers++;
//            statistic.fail_answers++;

            $(this).hide();

            $('div.question-choose .variant').removeClass('choosed');
        }
        recalculateScore();
    };

    /**
     * Answer is confirmed, run custom actions like animations
     * @param correct
     * @param answerIndex
     */

    liveFastClick('div.game a.button-question-continue', function () {
        var ans_div = $('div.game .step-4');
        if (ans_div.hasClass('incorrect')) {
            /*If we need to return to question if answer incorrectly do */
            questionIndex++;
            if (!empty(questions['q' + questionIndex])) {
                questionShow(questionIndex);
            } else {

                finishGame();
            }

            /* Else go to next question */
            /*questionIndex++;
             questionHide(questionIndex);
             recalculateScore();*/
        } else if (ans_div.hasClass('correct')) {
            questionIndex++;
            questionHide(questionIndex);
        }
    });


    /* -----------------------  STEP-5 Results ------------------------- */

    liveFastClick('div.game a.button-result-continue', function () {
        $('#game').removeClass('step-5');
        flashBackground('stop');
        /*
         game.resetAnimation();
         */
        game.start();
    });

    /* ----------------- ANIMATIONS ----------------- */

    /**
     * Init animation
     */
    this.prepareAnimationFrame = function () {
        arrow = $('div.game .arrow');

        $('div.game #animation').hide();
        var lightbulb = '';
        for (i = 0; i < 36; i++) {
            lightbulb += '<div id="light' + i + '" class="light"></div>';
        }

        $('div.game #animation .lightbulb').html(lightbulb);
        light_obj = $('div.game .light');
        moveR(95, 10, 36, 1, 248, 17, 228, 228, 212, 'light0', 100, 36);
    }

    var x, y, obj, corner;
    var pi = Math.PI;

    /**
     alfa - initial angle
     beta - at what angle to rotate
     dalfa - step angle
     ro - 0 the direction of rotation in a clockwise one counterclockwise
     x - coordinates of a point on the x-axis
     y - the coordinates of a point on the y-axis
     centre - coordinate of the center of the x-axis
     centrY - coordinate of the center along the y axis
     Radius - radius of the
     obid - id of the object
     timeout - time interval
     */
    function moveR(alfa, dalfa, beta, ro, x, y, centrX, centrY, Radius, obid, timeout, num) {

        if (alfa == 360 || alfa == -360) alfa = 0;
        obj = document.getElementById(obid);
        obj.style.left = x + "px";
        obj.style.top = y + "px";
        corner = 2 * pi - alfa / 180 * pi; // переводим градусы в радианы с учетом периода в 2 пи
        x = parseInt(Radius * Math.cos(corner) + centrX);  // считаем новые координаты точки по оси х
        y = parseInt(Radius * Math.sin(corner) + centrY);  // считаем новые координаты по оси у
        if (ro == 1) alfa += dalfa; else alfa -= dalfa; // определяем куда нам откладывать угол
        beta -= dalfa;
        num--;

        if (num > 0) moveR(alfa, dalfa, beta, ro, x, y, centrX, centrY, Radius, 'light' + num, timeout, num);
    }

    /**
     alfa - initial angle
     beta - at what angle to rotate
     dalfa - step angle
     ro - 0 the direction of rotation in a clockwise one counterclockwise
     timeout - time interval
     */

    function moveArrow(alfa, dalfa, beta, ro, timeout) {

        if (alfa == 360 || alfa == -360) alfa = 0;
        a = alfa;
        //corner=2*pi-alfa/180*pi;
        //if(ro==1) alfa+=dalfa; else alfa-=dalfa;
        alfa += dalfa
        beta -= dalfa;

        //a =  Math.abs(beta%360);
        //console.log ("a",a,"alfa", alfa, "beta", beta);
        /*if(a%60 == 0) {
         // tRewind(sounds.hover, PRIORITY_LOW);
         tPlay(sounds.hover, PRIORITY_LOW);
         setTimeout(function(){
         var l = (a/10);
         light_obj.removeClass('glow');
         for (i=0; i<6; i++){
         light_obj.eq(l+i).addClass('glow');
         }
         },timeout)
         }*/

        if ($('div.game .wheelspin div.start_button').hasClass('start') == true) {

            arrow.css({
                "-ms-Transform": 'rotate(' + (-beta) + 'deg)',
                "Transform": 'rotate(' + (-beta) + 'deg) '
            });

            setTimeout(function () {
                moveArrow(alfa, dalfa, beta, ro, timeout);
            }, timeout)
        } else {
            arrow.css({
                "-ms-Transform": 'rotate(' + (-beta) + 'deg)',
                "Transform": 'rotate(' + (-beta) + 'deg) '
            });
            if (counter_move_ie < 20) {
                counter_move_ie++
                dalfa = (((100 - 4 * counter_move_ie) / 10) | 0)
                setTimeout(function () {
                    moveArrow(alfa, dalfa, beta, ro, timeout);
                }, timeout)
            } else {

                beta -= 3 * dalfa;
                /*arrow.css({
                 "-ms-Transition-Duration": timeout*10+'ms',
                 "transition-duration":  timeout*10+'ms',
                 "-ms-Transform":'rotate('+(-beta)+'deg) ',
                 "Transform":'rotate('+(-beta)+'deg) ',
                 "-ms-Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)',
                 "Transition-Timing-Function": 'cubic-bezier(0, 0, 0.58, 1.0)'

                 }).data('run', [alfa,dalfa,beta]);*/
                //console.log("a1",a);
                a = (a < 340) ? a : 0;
                // console.log("a2",a);
                a = ( a % 60 == 0) ? a : (a % 60 > 30) ? a + (60 - a % 60) : a - (a % 60);
                //console.log("a3",a);

                /*setTimeout(function(){
                 var l = (a/10);
                 light_obj.removeClass('glow');
                 for (i=0; i<6; i++){
                 light_obj.eq(l+i).addClass('glow');
                 }
                 },timeout)*/

                clearInterval(timerClick);
                currentDeg = getCurrentRotateDeg();
                scoreQuestion = (currentDeg < 60 || currentDeg >= 240 && currentDeg < 300) ? 300 : (currentDeg >= 60 && currentDeg < 120 || currentDeg >= 180 && currentDeg < 240) ? 500 : 100;
                //console.log(scoreQuestion);
                $('#game').data('score', scoreQuestion);
                $('div.game .buttons-continue-wheel .button').addClass('show');
                $('div.game .wheelspin').addClass('not_hover');

            }
        }

    }


    function getCurrentRotateDeg(debu) { //дает текущий угол поворота - 0-360 градусов
        var prev = new Date().getTime();
        if (debu) console.log("start -", new Date().getTime());
        var st = window.getComputedStyle(arrow.get(0), null);
        if (debu) console.log("getComputedStyle -", new Date().getTime() - prev);
        prev = new Date().getTime();
        var tr = st.getPropertyValue("-webkit-transform") ||
            st.getPropertyValue("-moz-transform") ||
            st.getPropertyValue("-ms-transform") ||
            st.getPropertyValue("-o-transform") ||
            st.getPropertyValue("transform") ||
            "FAIL";
        if (debu)  console.log("getPropertyValue -", new Date().getTime() - prev);
        prev = new Date().getTime();

        var values = tr.split('(')[1].split(')')[0].split(',');
        if (debu) console.log("split-", new Date().getTime() - prev);
        prev = new Date().getTime();
        var a = values[0];//cos
        var b = values[1];//sin
        var current_Deg = Math.round(Math.atan2(b, a) * (180 / Math.PI));
        if (debu) console.log("current_Deg", new Date().getTime() - prev);
        prev = new Date().getTime();
        if (current_Deg < 0) {
            current_Deg += 360;
        }
        if (debu)  console.log("return_deg", new Date().getTime() - prev);
        prev = new Date().getTime();
        return current_Deg
    }


    /* ----------------- SCORE ------------------- */
    this.onUpdateScore = function (score) {
        $('div.game div.score').html(score);
    }

    /* -----------------------  GAME FLOW END ------------------------- */


    /* ----------------- GAME SPECIFIC LOADERS ------------------- */
    this.loadStyles = function () {
        applyDefaultStyles(design);

        /* eli sprite anims */
        loadEliSpriteAnimStyles(dynamicCssInstance, design);

        if (!hoverDisable) {
            dynamicCssInstance.addCompiled("div.game .question-choose .variant:hover", design.question_button_over);
            dynamicCssInstance.addCompiled("div.game a.button:hover", design.button_over);
        }

        dynamicCssInstance.addRule("div.game .question-vertical-shift", design['margin_top_for_questions_screen'], "height: $vpx");
        dynamicCssInstance.addRule("div.game .question-feedback-vertical-shift", design['margin_top_for_questions_feedback'], "height: $vpx");

        if (!empty(design.wheelspin)) {
            dynamicCssInstance.addCompiled("div.game .wheelspin", design.wheelspin);
        }

        dynamicCssInstance.addCompiled("div.game div.logo1", design.logo1);
        dynamicCssInstance.addCompiled("div.game div.logo2", design.logo2);
        dynamicCssInstance.addCompiled("div.game div.logo3", design.logo3);

        if (!empty(design.question_screen)) {
            var object = dozerMapper(design.question_screen, ["width", "height", "X", "Y", "padding", "paddingX", "paddingY", "margin", "marginX", "marginY", "marginTop", "marginBottom", "marginLeft", "marginRight"]);
            dynamicCssInstance.addCompiled("div.game .vertical .question-choose-wrapper", object);
            dynamicCssInstance.addCompiled("div.game .vertical .question-block-wrapper", object);
            dynamicCssInstance.addCompiled("div.game .vertical .question-answered-block-wrapper", object);
        }

        if (!empty(design.question_choose_wrapper)) {
            dynamicCssInstance.addCompiled("div.game .vertical .question-choose-wrapper", design.question_choose_wrapper);
        }
        dynamicCssInstance.addCompiled("div.game .question-choose .variant", design.question_button_up);
        dynamicCssInstance.addCompiled("div.game .question-choose .variant:active", design.question_button_down);
        dynamicCssInstance.addCompiled("div.game .question-choose .variant.choosed", design.question_button_selected);

        dynamicCssInstance.addCompiled("div.game a.button", design.button_up);
        dynamicCssInstance.addCompiled("div.game a.button:active", design.button_down);
        if (!empty(design.score_box)) {
            dynamicCssInstance.addCompiled("div.game div.score", design.score_box);
        }
        if (!empty(design.scoreboard_box)) {
            dynamicCssInstance.addCompiled("div.game div.scoreboard", design.scoreboard_box);
        }

        dynamicCssInstance.addCompiled("div.game .vertical .question-answered-block-wrapper", design.question_feedback_box);
        dynamicCssInstance.addCompiled("div.game .vertical .question-block-wrapper>div.question-block-wrapper-inner", design.question_box);
        var object = dozerMapper(design.question_box, ["width", "height", "X", "Y", "padding", "paddingX", "paddingY"]);
        dynamicCssInstance.addCompiled("div.game .question-block-wrapper>div.question-image", object);

        dynamicCssInstance.addCompiled("div.game div.step-2-description", design.description_panel);
        dynamicCssInstance.addCompiled("div.game .result-block-wrapper", design.result_panel);
        dynamicCssInstance.addCompiled("div.game .indicator", design.progressbar_container);
        dynamicCssInstance.addRule("div.game .step-4.answered.correct .question-answered-block-wrapper h1", design.question_answer_correct_color, "color: $v");
        dynamicCssInstance.addRule("div.game .step-4.answered.incorrect .question-answered-block-wrapper h1", design.question_answer_incorrect_color, "color: $v");

        dynamicCssInstance.addRule("div.game .progressbar>div", design.progressbar_container.spacing, "margin-right: $vpx");
        dynamicCssInstance.addRule("div.game .progressbar>div:last-child", design.progressbar_container.spacing, "margin-right: 0px");

        $('div.game .step-4').addClass('vertical').removeClass('horizontal');


        dynamicCssInstance.addCompiled("div.game #step1continuebutton", design.splash_continuebutton);
        dynamicCssInstance.addCompiled("div.game #step2continuebutton", design.intro_continuebutton);
        dynamicCssInstance.addCompiled("div.game #step4continuebutton", design.question_continuebutton);
        dynamicCssInstance.addCompiled("div.game #step4confirmbutton", design.question_confirmbutton);
        dynamicCssInstance.addCompiled("div.game #step5replaybutton", design.results_replaybutton);

        var qCount = 1;
        while (!empty(questions['q' + qCount])) {
            qCount++;
        }
        qCount--;

        var paddingTMP = design.progressbar_container.paddingX || design.progressbar_container.padding || 0;
        var totalWidth = 1 * value(design.progressbar_container.width) - qCount * value(design.progressbar_container.spacing) - 2 * paddingTMP;
        var elementWidth = Math.floor(totalWidth / qCount);
        dynamicCssInstance.addRule("div.game .progressbar>div", elementWidth, "width: $vpx");

        dynamicCssInstance.addRule("div.game .progressbar>div", design.progressbar_item.width, "width: $vpx");


        dynamicCssInstance.flush();
    }
    this.loadSounds = function () {
        if (questions.introduction_audio != null) {
            var media = createSound(questions.introduction_audio, true);

            sounds.introduction = media;
        }
        if (questions.conclusion_audio != null) {
            var media = createSound(questions.conclusion_audio, true);

            sounds.conclusion = media;
        }
        if (onlyOneSound) {
            liveFastClick('.game a:not(#step4confirmbutton)', function () {
                tPlay(sounds.select, PRIORITY_LOW);
            });
            liveFastClick('.game .questions div.question:not(.answered):not(.hasOwnSound)', function () {
                tPlay(sounds.select, PRIORITY_LOW);
            });
            liveFastClick('.type-multiple .question-choose .variant', function () {
                tPlay(sounds.select, PRIORITY_LOW);
            });
        } else {
            liveFastClick('.game a, .game .questions div.question:not(.answered), .question-choose .variant', function () {
                tPlay(sounds.select, PRIORITY_LOW);
            });
            if (!hoverDisable) {
                $('.game a, .game .questions div.question:not(.answered), div.game .question-choose .variant, .wheelspin:not(.not_hover) .start_button').live('mouseenter', function () {
                    tPlay(sounds.hover, PRIORITY_LOW);
                });
            }
        }
    }
    this.loadData = function () {
        questions.randomize_question_order = stringToBoolean(questions.randomize_question_order);
        questions.randomize_answer_order = stringToBoolean(questions.randomize_answer_order);


        var i = 1;
        while (!empty(questions['q' + i])) {
            i++;
        }
        var qlength = i - 1;
        if (questions.randomize_question_order) {
            var newQ = [];
            var oldQ = [];
            var i = 1;
            while (!empty(questions['q' + i])) {
                newQ[newQ.length] = questions['q' + i];
                oldQ[oldQ.length] = empty(questions['q' + i].path) ? (((i) * 100 / qlength).toFixed(2) + "%") : questions['q' + i].path;
                i++;
            }

            newQ.sort(function () {
                return 0.5 - Math.random()
            });
            var ql = i;
            for (i = 1; i < ql; i++) {
                questions['q' + i] = newQ[i - 1];
                questions['q' + i].path = oldQ[i - 1];
            }

        }

        /* FILL GAME TEXT */
        $("div.game #step1continuebutton").html("" + value(questions.splash_page_button_continue_text));
        $("div.game #step2continuebutton").html("" + value(questions.intro_page_button_continue_text));
        $("div.game #step3continuebutton").html("" + value(questions.questions_page_button_continue_text));
        $("div.game #step4continuebutton").html("" + value(questions.question_page_button_continue_text));
        $("div.game #step4wheelbutton").html("" + value(questions.wheel_page_button_continue_text));
        $("div.game #step4confirmbutton").html("" + value(questions.question_page_button_confirm_text));
        $("div.game #step5replaybutton").html("" + value(questions.result_page_button_replay_text));
        //conditionalShow($('div.step-2-description h1'), questions.introduction_title);
        $('div.game div.step-2-description div.description div').html(value(questions.introduction_text));
        //$('div.step-2-description div.racer-description').html("" + value(questions.introduction_racer_description));

        //$('div.result-block h1').html("" + value(questions.conclusion_title));
        $('div.game div.result-block div.description').html(value(questions.conclusion_text));
        /*         conditionalShow($('div.result-block div.score-description'), questions.finish_score_description); */


        var qCount = 1;
        while (!empty(questions['q' + qCount])) {
            qCount++;
        }
        questionCount = qCount;

        questionCount--;

    }
};

$(document).ready(function () {
    game.readConfig();
    $('.game').css('opacity', 0.1);
    $('div.game #animation').hide();

});

$(window).load(function () {

});

$(document).bind('gameLoaded', function () {
    SCOPreInitialize();
    SCOInitialize();
    $('.game').css('opacity', 1);

    game.start();
});